const path = require('path');
const webpack = require('webpack');
const StatsPlugin = require('stats-webpack-plugin');

// must match config.webpack.dev_server.port
const devServerPort = 4808;

// set TARGET=production on the environment to add asset fingerprints
const production = process.env.TARGET === 'production';

const webpackPath = (directory) => (
  path.join(__dirname, '.', `src/${directory}`)
);

const config = {
  output: {
    // Build assets directly in to public/webpack/, let webpack know
    // that all webpacked assets start with webpack/

    // must match config.webpack.output_dir
    path: path.join(__dirname, '.', 'public', 'src'),
    publicPath: '/src/',

    filename: production ? '[name]-[chunkhash].js' : '[name].js'
  },

  module: {
    loaders: [
      {
        test: /\.js$/,
        exclude: path.resolve(__dirname, 'node_modules'),
        loader: 'babel'
      },
      {
        test: /\.json$/,
        loader: 'json'
      },
      {
        test: /\.scss$/,
        loader: 'style!css!postcss!sass!themed-sass?sourceMap&theme=' + process.env.TENANT
      },
      {
        test: /\.css$/,
        loader: 'style!css!postcss?sourceMap'
      }
    ]
  },

  resolve: {
    root: [
      path.join(__dirname, '.', 'src'),
    ],
    alias: {
      actions: webpackPath('actions'),
      api: webpackPath('api'),
      components: webpackPath('components'),
      constants: webpackPath('constants'),
      containers: webpackPath('containers'),
      reducers: webpackPath('reducers'),
      sagas: webpackPath('sagas'),
      store: webpackPath('store')
    }
  },

  plugins: [
    // must match config.webpack.manifest_filename
    new StatsPlugin('manifest.json', {
      // We only need assetsByChunkName
      chunkModules: false,
      source: false,
      chunks: false,
      modules: false,
      assets: true
    })]
};

if (production) {
  config.entry = {
    // Sources are expected to live in $app_root/webpack
    index: [
      './src/standalone.js'
    ]
  };

  config.plugins.push(
    new webpack.NoErrorsPlugin(),
    new webpack.optimize.UglifyJsPlugin({
      compressor: { warnings: false },
      sourceMap: false
    }),
    new webpack.DefinePlugin({
      'process.env': { NODE_ENV: JSON.stringify('production') }
    }),
    new webpack.optimize.OccurenceOrderPlugin()
  );
} else {
  config.entry = {
    // Sources are expected to live in $app_root/webpack
    index: [
      'webpack-dev-server/client?http://localhost:4808',
      'webpack/hot/only-dev-server',
      './src/demo.js'
    ]
  };

  config.devServer = {
    hot: true,
    port: devServerPort,
    headers: { 'Access-Control-Allow-Origin': '*' }
  };
  config.plugins.push(new webpack.HotModuleReplacementPlugin());
  config.output.publicPath = '//localhost:' + devServerPort + '/public/';
  // Source maps
  config.devtool = '#cheap-module-eval-source-map';
}

module.exports = config;
